package com.css.simulation.resource.scheduler.scheduler;

import api.common.pojo.constants.DictConstants;
import api.common.util.CollectionUtil;
import api.common.util.LinuxUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.scheduler.mapper.ProjectMapper;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.pojo.po.TaskPO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.List;

@Component
@Slf4j
public class TickScheduler {

    @Value("${scheduler.manual-project.topic}")
    String manualProjectTopic;
    @Autowired
    StringRedisTemplate redisTemplate;

    @Autowired
    TaskMapper taskMapper;
    @Autowired
    ProjectMapper projectMapper;

    @Scheduled(fixedDelay = 2000)
    public void tick() {

        List<TaskPO> executingTaskList = taskMapper.selectExecuting();
        if (CollectionUtil.isEmpty(executingTaskList)) {
            return;
        }
//        log.info("------- TickScheduler 查询出所有执行中的任务（'Running'）:" + executingTaskList);
        //2 根据 key 查出任务的心跳时间
        executingTaskList.forEach(task -> {
            String taskId = task.getId();
            String projectId = task.getPId();
            try {
                String s = redisTemplate.opsForValue().get(manualProjectTopic + ":" + projectId + ":" + taskId);
//                Optional.ofNullable(s).orElseThrow(() -> new RuntimeException("项目 " + projectId + " 下的任务 " + taskId + " 的心跳查询失败"));
                assert s != null;
                long tickTime = Long.parseLong(s);
                long maxSimulationTime = task.getMaxSimulationTime() * 1000;
                long now = TimeUtil.getNow();
                long difference = now - tickTime;
//                log.info("------- TickScheduler 任务" + taskId + "心跳时间为：" + tickTime + "最大仿真时间为：" + tickTime + "时间差为：" + difference);
                if (difference > maxSimulationTime) {
//                    log.info("------- TickScheduler 任务" + taskId + "已超时，状态修改为：" + DictConstants.TASK_ABORTED);
                    taskMapper.updateState(taskId, DictConstants.TASK_ABORTED);
                }
            } catch (Exception e) {
                throw new RuntimeException(e.getMessage());
            }

        });

    }


    /**
     * 检查如果有 job 在运行但是 pod 全部关闭的情况，此时需要重启一下 job
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void checkProject() {
        //1 查询出正在运行中的 project
        List<String> projectIdList = projectMapper.selectIdByState("20");
        //2 根据 projectId 获取 pod
        projectIdList.forEach(projectId -> {
            try {
                String execute = LinuxUtil.execute("kubectl get pod | grep project-" + projectId);
                int i = StringUtil.countSubString(execute, projectId);
                if (i == 0) {
                    LinuxUtil.execute("kubectl delete job project-" + projectId);
                    String jobTemplateYamlPathTarget = "/opt/simulation-cloud/simulation-resource-scheduler/job-yaml/" + "project-" + projectId + ".yaml";
                    LinuxUtil.execute("kubectl apply -f " + jobTemplateYamlPathTarget);
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        });

        //3 如果 pod 为空，则重启 job

    }
}
